<?php

// File: src/Form/DynamicImageGeneratorSettingsForm.php

namespace Drupal\dynamic_image_generator\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Configure Dynamic Image Generator API settings.
 * 
 * This form handles API configuration, different from ImageTemplateSettingsForm
 * which handles template entity settings.
 */
class DynamicImageGeneratorSettingsForm extends ConfigFormBase {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a DynamicImageGeneratorSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $http_client, MessengerInterface $messenger, LoggerChannelFactoryInterface $logger_factory) {
    parent::__construct($config_factory);
    $this->httpClient = $http_client;
    $this->messenger = $messenger;
    $this->logger = $logger_factory->get('dynamic_image_generator');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('messenger'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['dynamic_image_generator.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'dynamic_image_generator_api_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('dynamic_image_generator.settings');

    // Page header
    $form['header'] = [
      '#type' => 'markup',
      '#markup' => '<div class="image-settings-header">
        <h2>🎨 Dynamic Image Generator API Settings</h2>
        <p>Configure your image generation API credentials and settings.</p>
      </div>',
    ];

    // API Configuration Section
    $form['api_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('🔧 API Configuration'),
      '#open' => TRUE,
      '#description' => $this->t('Configure your HTML/CSS to Image API credentials. Get your credentials from <a href="@url" target="_blank">htmlcsstoimage.com dashboard</a>.', [
        '@url' => 'https://htmlcsstoimage.com/dashboard',
      ]),
    ];

    $form['api_settings']['api_provider'] = [
      '#type' => 'select',
      '#title' => $this->t('API Provider'),
      '#description' => $this->t('Select your image generation API provider.'),
      '#options' => [
        'htmlcsstoimage' => $this->t('HTML/CSS to Image API (htmlcsstoimage.com)'),
        'bannerbear' => $this->t('Bannerbear API (coming soon)'),
        'placid' => $this->t('Placid API (coming soon)'),
      ],
      '#default_value' => $config->get('api_provider') ?: 'htmlcsstoimage',
      '#required' => TRUE,
    ];

    $form['api_settings']['api_user_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API User ID'),
      '#description' => $this->t('Your API User ID from htmlcsstoimage.com dashboard.'),
      '#default_value' => $config->get('api_user_id'),
      '#required' => TRUE,
      '#size' => 60,
      '#maxlength' => 255,
      '#placeholder' => 'e.g., 1b860f7f-c7b0-4daf-a46b-3f086fe6aade',
    ];

    $form['api_settings']['api_key'] = [
      '#type' => 'password',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('Your API Key from htmlcsstoimage.com dashboard. Leave blank to keep current key.'),
      '#size' => 60,
      '#maxlength' => 255,
      '#placeholder' => 'Enter your API key here...',
    ];

    // Show current API key status
    if ($config->get('api_key')) {
      $form['api_settings']['current_key_status'] = [
        '#type' => 'markup',
        '#markup' => '<div class="messages messages--status">
          <strong>✅ API Key Status:</strong> Currently configured (last 8 characters: ••••••••' . substr($config->get('api_key'), -8) . ')
          <br><small>Enter a new key above to update it, or leave blank to keep the current key.</small>
        </div>',
      ];
    }

    $form['api_settings']['api_endpoint'] = [
      '#type' => 'url',
      '#title' => $this->t('API Endpoint'),
      '#description' => $this->t('The API endpoint URL. Usually you don\'t need to change this.'),
      '#default_value' => $config->get('api_endpoint') ?: 'https://hcti.io/v1/image',
      '#required' => TRUE,
      '#size' => 80,
    ];

    // Image Generation Settings
    $form['generation_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('🖼️ Image Generation Settings'),
      '#open' => TRUE,
    ];

    $form['generation_settings']['default_width'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Image Width'),
      '#description' => $this->t('Default width in pixels. Recommended: 1200px for social media.'),
      '#default_value' => $config->get('default_width') ?: 1200,
      '#min' => 100,
      '#max' => 5000,
      '#field_suffix' => 'px',
      '#required' => TRUE,
    ];

    $form['generation_settings']['default_height'] = [
      '#type' => 'number',
      '#title' => $this->t('Default Image Height'),
      '#description' => $this->t('Default height in pixels. Recommended: 630px for social media.'),
      '#default_value' => $config->get('default_height') ?: 630,
      '#min' => 100,
      '#max' => 5000,
      '#field_suffix' => 'px',
      '#required' => TRUE,
    ];

    $form['generation_settings']['image_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Default Image Format'),
      '#description' => $this->t('The format for generated images.'),
      '#options' => [
        'png' => $this->t('PNG - Best quality, supports transparency'),
        'jpg' => $this->t('JPG - Good quality, smaller file size'),
        'webp' => $this->t('WebP - Modern format, excellent compression'),
      ],
      '#default_value' => $config->get('image_format') ?: 'png',
      '#required' => TRUE,
    ];

    $form['generation_settings']['quality'] = [
      '#type' => 'range',
      '#title' => $this->t('Image Quality'),
      '#description' => $this->t('Image quality from 1-100. Only applies to JPG and WebP formats.'),
      '#default_value' => $config->get('quality') ?: 90,
      '#min' => 1,
      '#max' => 100,
      '#step' => 1,
      '#field_suffix' => '%',
      '#states' => [
        'visible' => [
          ':input[name="image_format"]' => [
            ['value' => 'jpg'],
            ['value' => 'webp'],
          ],
        ],
      ],
    ];

    // Performance Settings
    $form['performance_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('⚡ Performance Settings'),
      '#open' => FALSE,
    ];

    $form['performance_settings']['request_timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('API Request Timeout'),
      '#description' => $this->t('Maximum time to wait for API response in seconds.'),
      '#default_value' => $config->get('request_timeout') ?: 60,
      '#min' => 10,
      '#max' => 300,
      '#field_suffix' => 'seconds',
    ];

    $form['performance_settings']['retry_attempts'] = [
      '#type' => 'number',
      '#title' => $this->t('Retry Attempts'),
      '#description' => $this->t('Number of times to retry failed API requests.'),
      '#default_value' => $config->get('retry_attempts') ?: 2,
      '#min' => 0,
      '#max' => 5,
    ];

    $form['performance_settings']['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Debug Mode'),
      '#description' => $this->t('Log detailed API information. Useful for troubleshooting.'),
      '#default_value' => $config->get('debug_mode') ?: FALSE,
    ];

    // Testing Section
    $form['testing'] = [
      '#type' => 'details',
      '#title' => $this->t('🧪 API Testing'),
      '#open' => FALSE,
    ];

    $form['performance_settings']['#access'] = FALSE;
    $form['generation_settings']['#access'] = FALSE;

    $form['testing']['test_connection'] = [
      '#type' => 'submit',
      '#value' => $this->t('🔍 Test API Connection'),
      '#submit' => ['::testApiConnection'],
      '#validate' => ['::validateApiCredentials'],
      '#limit_validation_errors' => [
        ['api_user_id'],
        ['api_key'],
        ['api_endpoint'],
      ],
      '#attributes' => [
        'class' => ['button', 'button--primary'],
      ],
    ];

    // Add styling
    $form['#attached']['html_head'][] = [
      [
        '#type' => 'html_tag',
        '#tag' => 'style',
        '#value' => '
          .image-settings-header {
            background: linear-gradient(135deg, #f8f9fa 0%, #e9ecef 100%);
            padding: 20px;
            border-radius: 8px;
            margin-bottom: 20px;
            border-left: 4px solid #007bff;
          }
          .image-settings-header h2 {
            margin: 0 0 10px 0;
            color: #2c3e50;
          }
        ',
      ],
      'image_api_settings_css',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Validates API credentials for testing.
   */
  public function validateApiCredentials(array &$form, FormStateInterface $form_state) {
    $config = $this->config('dynamic_image_generator.settings');
    
    $user_id = $form_state->getValue('api_user_id') ?: $config->get('api_user_id');
    $api_key = $form_state->getValue('api_key') ?: $config->get('api_key');

    if (empty($user_id)) {
      $form_state->setErrorByName('api_user_id', $this->t('API User ID is required for testing.'));
    }

    if (empty($api_key)) {
      $form_state->setErrorByName('api_key', $this->t('API Key is required for testing.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('dynamic_image_generator.settings');
    
    // Save all settings
    $config->set('api_provider', $form_state->getValue('api_provider'))
      ->set('api_user_id', $form_state->getValue('api_user_id'))
      ->set('api_endpoint', $form_state->getValue('api_endpoint'))
      ->set('default_width', $form_state->getValue('default_width'))
      ->set('default_height', $form_state->getValue('default_height'))
      ->set('image_format', $form_state->getValue('image_format'))
      ->set('quality', $form_state->getValue('quality'))
      ->set('request_timeout', $form_state->getValue('request_timeout'))
      ->set('retry_attempts', $form_state->getValue('retry_attempts'))
      ->set('debug_mode', $form_state->getValue('debug_mode'));

    // Only update API key if a new one was provided
    $api_key = $form_state->getValue('api_key');
    if (!empty($api_key)) {
      $config->set('api_key', $api_key);
    }

    $config->save();

    $this->messenger->addMessage($this->t('✅ Dynamic Image Generator API settings have been saved successfully.'));
    
    parent::submitForm($form, $form_state);
  }

  /**
   * Test API connection submit handler.
   */
  public function testApiConnection(array &$form, FormStateInterface $form_state) {
    $config = $this->config('dynamic_image_generator.settings');
    
    $user_id = $form_state->getValue('api_user_id') ?: $config->get('api_user_id');
    $api_key = $form_state->getValue('api_key') ?: $config->get('api_key');
    $endpoint = $form_state->getValue('api_endpoint') ?: $config->get('api_endpoint');

    try {
      $test_data = [
        'html' => '<div style="width: 300px; height: 150px; background: #007bff; color: white; display: flex; align-items: center; justify-content: center; font-family: Arial; font-size: 24px; font-weight: bold;">✅ API Test Success</div>',
        'css' => 'body { margin: 0; padding: 20px; }',
        'width' => 340,
        'height' => 190,
      ];

      $response = $this->httpClient->request('POST', $endpoint, [
        'form_params' => $test_data,
        'auth' => [$user_id, $api_key],
        'timeout' => 30,
      ]);

      if ($response->getStatusCode() === 200) {
        $result = json_decode($response->getBody(), TRUE);
        if (isset($result['url'])) {
          $this->messenger->addMessage($this->t('✅ <strong>API Connection Successful!</strong><br>🖼️ Test image: <a href="@url" target="_blank">View Test Image</a>', [
            '@url' => $result['url'],
          ]));
        }
      }
    } catch (\Exception $e) {
      $this->messenger->addError($this->t('❌ API connection failed: @error', [
        '@error' => $e->getMessage(),
      ]));
    }
  }
}